@@ -1,20 +1,10 @@
--- ignore warning from 'import Data.Text.Encoding'
-{-# OPTIONS_GHC -fno-warn-unused-imports #-}
-
-{-# LANGUAGE CPP #-}
-
-#ifdef USE_MONO_PAT_BINDS
-{-# LANGUAGE MonoPatBinds #-}
-#endif
-
+------------------------------------------------------------------------------
 -- |
--- Module      : Blaze.ByteString.Builder.Char8
--- Copyright   : (c) 2010 Simon Meier
--- License     : BSD3-style (see LICENSE)
---
--- Maintainer  : Simon Meier <iridcode@gmail.com>
--- Stability   : experimental
--- Portability : tested on GHC only
+-- Module:      Blaze.ByteString.Builder.Char8
+-- Copyright:   (c) 2013 Leon P Smith
+-- License:     BSD3
+-- Maintainer:  Leon P Smith <leon@melding-monads.com>
+-- Stability:   experimental
 --
 -- //Note:// This package is intended for low-level use like implementing
 -- protocols. If you need to //serialize// Unicode characters use one of the
@@ -25,6 +15,8 @@
 -- This corresponds to what the 'bytestring' package offer in
 -- 'Data.ByteString.Char8'.
 --
+------------------------------------------------------------------------------
+
 module Blaze.ByteString.Builder.Char8
     (
       -- * Writing Latin-1 (ISO 8859-1) encodable characters to a buffer
@@ -38,47 +30,40 @@ module Blaze.ByteString.Builder.Char8
     , fromLazyText
     ) where
 
-import Foreign
-import Data.Char (ord)
-
-import qualified Data.Text               as TS
-import qualified Data.Text.Encoding      as TS -- imported for documentation links
-import qualified Data.Text.Lazy          as TL
-import qualified Data.Text.Lazy.Encoding as TS -- imported for documentation links
-
-import Blaze.ByteString.Builder.Internal
-import Blaze.ByteString.Builder.Word
+import Blaze.ByteString.Builder.Compat.Write ( Write, writePrimFixed )
+import           Data.ByteString.Builder ( Builder )
+import qualified Data.ByteString.Builder as B
+import qualified Data.ByteString.Builder.Prim as P
+import qualified Data.Text      as TS
+import qualified Data.Text.Lazy as TL
 
 -- | Write the lower 8-bits of a character to a buffer.
---
-{-# INLINE writeChar #-}
 writeChar :: Char -> Write
-writeChar = writeWord8 . fromIntegral . ord
+writeChar = writePrimFixed P.char8
+{-# INLINE writeChar #-}
 
 -- | /O(1)/. Serialize the lower 8-bits of a character.
---
 fromChar :: Char -> Builder
-fromChar = fromWriteSingleton writeChar
+fromChar = B.char8
+{-# INLINE fromChar #-}
 
 -- | /O(n)/. Serialize the lower 8-bits of all characters of a string
---
 fromString :: String -> Builder
-fromString = fromWriteList writeChar
+fromString = P.primMapListFixed P.char8
+{-# INLINE fromString #-}
 
 -- | /O(n)/. Serialize a value by 'Show'ing it and serializing the lower 8-bits
 -- of the resulting string.
---
 fromShow :: Show a => a -> Builder
 fromShow = fromString . show
+{-# INLINE fromShow #-}
 
 -- | /O(n)/. Serialize the lower 8-bits of all characters in the strict text.
---
-{-# INLINE fromText #-}
 fromText :: TS.Text -> Builder
 fromText = fromString . TS.unpack
+{-# INLINE fromText #-}
 
 -- | /O(n)/. Serialize the lower 8-bits of all characters in the lazy text.
---
-{-# INLINE fromLazyText #-}
 fromLazyText :: TL.Text -> Builder
 fromLazyText = fromString . TL.unpack
+{-# INLINE fromLazyText #-}
