@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, BangPatterns, MonoPatBinds #-}
+{-# LANGUAGE CPP, BangPatterns #-}
 
 -- |
 -- Module      : Blaze.ByteString.Builder.Internal.Poke
@@ -49,12 +49,14 @@ module Blaze.ByteString.Builder.Internal.Write (
 
 import Foreign
 
-import Data.Monoid
-
 import Control.Monad
 
 import Data.ByteString.Builder.Internal
 
+#if !MIN_VERSION_base(4,8,0)
+import Data.Monoid
+#endif
+
 ------------------------------------------------------------------------------
 -- Poking a buffer and writing to a buffer
 ------------------------------------------------------------------------------
